/*-
 * Copyright (C) GD <gd@pikenet.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: num2bit.c,v 1.1 2006/03/07 12:10:46 gd Exp $
 */

#include <postgres.h>
#include <utils/varbit.h>

PG_FUNCTION_INFO_V1(num2bit);

Datum 
num2bit(PG_FUNCTION_ARGS)
{
	VarBit         *newval, *oldval;
	uint32          port;
	int             rlen, newlen, ipad;
	bits8          *r, mask;

	oldval = PG_GETARG_VARBIT_P(0);

	if (PG_ARGISNULL(1))
		PG_RETURN_VARBIT_P(oldval);

	port = PG_GETARG_INT32(1);
	newlen = (port + 1 >= VARBITLEN(oldval)) ? port + 1 : VARBITLEN(oldval);

	rlen = VARBITTOTALLEN(newlen);
	newval = (VarBit *) palloc0(rlen);
	VARATT_SIZEP(newval) = rlen;
	VARBITLEN(newval) = newlen;

	memcpy(VARBITS(newval), VARBITS(oldval),
	       (VARBITBYTES(newval) <= VARBITBYTES(oldval)) ? VARBITBYTES(newval) : VARBITBYTES(oldval));

	r = VARBITS(newval);

	for (; port + 1 > BITS_PER_BYTE; r++, port -= BITS_PER_BYTE);
	*r |= (bits8) (1 << (BITS_PER_BYTE - (port + 1)));

	ipad = VARBITPAD(newval);
	if (ipad > 0) {
		mask = BITMASK << ipad;
		*(VARBITS(newval) + VARBITBYTES(newval) - 1) &= mask;
	}
	PG_RETURN_VARBIT_P(newval);
}
